/*
 * Author: Magnus Bergman <magnus@snisurset.net>
 *
 * This file is public domain.
 */
#ifndef eventloop_h
#define eventloop_h

#ifdef USE_GLIB
//# define EVENTLOOP_USE_GLIB
#endif

#include "dispatch.h"
#include "worker.h"

typedef int(*EventloopFdFunc)(void*,int,int);
typedef int(*EventloopTimeoutFunc)(void*);

#ifdef EVENTLOOP_USE_GLIB

#include <glib.h>

#define EVENTLOOP_IN  G_IO_IN
#define EVENTLOOP_OUT G_IO_OUT

typedef GMainLoop Eventloop;

#define eventloop_new() g_main_loop_new(NULL,FALSE)
#define eventloop_free(a) g_main_loop_unref(a)

#else /* EVENTLOOP_USE_GLIB */

#define EVENTLOOP_IN  1
#define EVENTLOOP_OUT 2

typedef struct _Eventloop Eventloop;

//int eventloop_init(Eventloop*);
//void eventloop_done(Eventloop*);
Eventloop* eventloop_new(void);
void eventloop_free(Eventloop*);

#endif /* EVENTLOOP_USE_GLIB */

void eventloop_add_fd(Eventloop*,int fd,int events,EventloopFdFunc,void*);
void eventloop_add_timeout(Eventloop*,int timout,EventloopTimeoutFunc,void*);
int eventloop_run(Eventloop*);
void eventloop_quit(Eventloop*);

int eventloop_other_thread(Eventloop*);
void eventloop_dispatch(Eventloop*,DispatchFunc,void*);
Worker* eventloop_create_worker(Eventloop*);

#endif /* eventloop_h */
