/*
 * Copyright (C) 2018-2019  Magnus Bergman <magnus@snisurset.net>
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, see <http:/www.gnu.org/licenses/>
 */
#include <X11/keysym.h>

#include "debug.h"
#include "keyboard.h"
#include "keyboard-xlib.h"

int keyboard_xlib_key(XKeyEvent* ev,XIC c){
    char buffer[32];
    KeySym key;
    int bytes;
    int i;
#ifdef X_HAVE_UTF8_STRING
    bytes = Xutf8LookupString(c,ev,buffer,32,&key,NULL);
#else
    bytes = XLookupString(ev,buffer,32,&key);
#endif
    if(bytes>0){
        for(i=0;i<bytes;++i){
            keyboard_key(buffer[i]);
        }
    }else{
        DEBUG_p("got %lu",key);
        switch(key){
        case XK_Up:
            keyboard_key(KEYBOARD_UP);
            break;
        case XK_Down:
            keyboard_key(KEYBOARD_DOWN);
            break;
        case XK_Left:
            keyboard_key(KEYBOARD_LEFT);
            break;
        case XK_Right:
            keyboard_key(KEYBOARD_RIGHT);
            break;
        case XK_Home:
            keyboard_key(KEYBOARD_HOME);
            break;
        case XK_End:
            keyboard_key(KEYBOARD_END);
            break;
        case XK_Page_Up:
            keyboard_key(KEYBOARD_PAGEUP);
            break;
        case XK_Page_Down:
            keyboard_key(KEYBOARD_PAGEDOWN);
            break;
        }
    }
    return 0;
}
