/*
 * Author: Magnus Bergman <magnus@snisurset.net>
 *
 * This file is public domain.
 */
#ifndef queue_h
#define queue_h

#ifdef USE_GLIB
//# define QUEUE_USE_GLIB
#endif

#ifdef QUEUE_USE_GLIB

#include <glib.h>

typedef GQueue Queue;
typedef GDestroyNotify QueueNotifyFunc;
typedef GCompareFunc QueueCompFunc;
typedef GFunc QueueForeachFunc;

#define queue_new()                g_queue_new()
#define queue_free(a)              g_queue_free(a)
#define queue_free_full(a,b)       g_queue_free_full(a,b)
#define queue_clear(a)             g_queue_clear(a)
#define queue_clear_full(a,b)      {g_queue_foreach(a,(GFunc)b,NULL);g_queue_clear(a);}
#define queue_push_head(a,b)       g_queue_push_head(a,b)
#define queue_push_tail(a,b)       g_queue_push_tail(a,b)
#define queue_pop_head(a)          g_queue_pop_head(a)
#define queue_pop_tail(a)          g_queue_pop_tail(a)
#define queue_insert_before(a,b,c) g_queue_insert_before(a,b,c)
#define queue_insert_after(a,b,c)  g_queue_insert_after(a,b,c)
#define queue_remove(a,b)          g_queue_delete_link(a,b)
#define queue_find_custom(a,b,c)   g_queue_find_custom(a,b,c)
#define queue_foreach(a,b,c)       g_queue_foreach(a,b,c)
#define queue_length(a)            g_queue_get_length(a)

#else

#include "list.h"
#include "memutil.h"

typedef struct{
    List* head;
    List* tail;
    int length;
}Queue;

typedef ListNotifyFunc QueueNotifyFunc;
typedef ListCompFunc QueueCompFunc;
typedef ListForeachFunc QueueForeachFunc;

#define queue_new() u_new0(Queue,1)
void    queue_free(Queue*);
void    queue_free_full(Queue*,ListNotifyFunc);
void    queue_clear(Queue*);
void    queue_clear_full(Queue*,ListNotifyFunc);
List*   queue_push_head(Queue*,void* data);
List*   queue_push_tail(Queue*,void* data);
void*   queue_pop_head(Queue*);
void*   queue_pop_tail(Queue*);
List*   queue_insert_before(Queue*,List*,void*);
List*   queue_insert_after(Queue*,List*,void*);
void*   queue_remove(Queue*,List*);
List*   queue_find_data(Queue*,void*);
List*   queue_find_custom(Queue*,void*,QueueCompFunc);
#define queue_foreach(a,b,c) list_foreach((a)->head,b,c)
#define queue_length(a) ((a)->length)

#endif /* QUEUE_USE_GLIB */

#define queue_prepend(a,b) queue_push_head(a,b)
#define queue_append(a,b) queue_push_tail(a,b)

#endif /* queue_h */
