/*
 * Author: Magnus Bergman <magnus@snisurset.net>
 *
 * This file is public domain.
 */
#ifndef thread_h
#define thread_h

#ifdef USE_GLIB
# define THREAD_USE_GLIB
#endif

#ifdef THREAD_USE_GLIB

#include <glib.h>

typedef GThread* Thread;

#define thread_dispatch(a,b) g_thread_new(NULL,(GThreadFunc)a,b)
#define thread_join(a) g_thread_join(a)
#define thread_self(a) g_thread_self(a)

typedef GMutex Lock;

#define lock_init(a) g_mutex_init(a)
#define lock_done(a) g_mutex_clear(a)
#define lock_lock(a) g_mutex_lock(a)
#define lock_unlock(a) g_mutex_unlock(a)

typedef GCond Condition;

#define condition_init(a) g_cond_init(a)
#define condition_done(a) g_cond_clear(a)
#define condition_wait(a,b) g_cond_wait(a,b)
#define condition_signal(a) g_cond_signal(a)
#define condition_broadcast(a) g_cond_broadcast(a)

#else

#include <pthread.h>

#include "dispatch.h"

typedef pthread_t Thread;

Thread thread_dispatch(DispatchFunc,void*);
void thread_join(Thread);
#define thread_self(a) pthread_self(a)

typedef pthread_mutex_t Lock;

#define lock_init(a) pthread_mutex_init(a,NULL)
#define lock_done(a) pthread_mutex_destroy(a)
#define lock_lock(a) pthread_mutex_lock(a)
#define lock_unlock(a) pthread_mutex_unlock(a)

typedef pthread_cond_t Condition;

#define condition_init(a) pthread_cond_init(a,NULL)
#define condition_done(a) pthread_cond_destroy(a)
#define condition_wait(a,b) pthread_cond_wait(a,b)
#define condition_signal(a) pthread_cond_signal(a)
#define condition_broadcast(a) pthread_cond_broadcast(a)

#endif

#endif /* thread_h */
